<#
.Synopsis
    Konvertiert einen exportierten Click-TT Spielplan in ein Google Kalender kompatibles CSV Format

.Description
    Konvertiert einen exportierten Click-TT Spielplan in ein Google Kalender kompatibles CSV Format

.Parameter SpielPlan
    Pfad zur Spielplan CSV Datei

.Parameter Spielzeit
    Optional: Wenn angegeben dann wird diese Zeit als Terminlnge (Stunden) benutzt. Standardwert: 3

.Example
    PS C:\> .\Convert-Plan.ps1 -Spielplan .\Vereinsspielplan_20160907135820.csv
    
    Description
    -----------

    Konvertiert den angegebenen Spielplan mit einer angenommenen Spielzeit von 3 Stunden

.Example
    PS C:\> .\Convert-Plan.ps1 -Spielplan .\Vereinsspielplan_20160907135820.csv -Spielzeit 4
    
    Description
    -----------

    Konvertiert den angegebenen Spielplan, aber verwendet eine Spieldauer von 4 Stunden anstatt 3 Stunden
#>
param(                  
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        [string]$Spielplan,
        [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true)]
        [int]$Spielzeit = 3
)

<#
    Function Declarations
#>
function Parse-Data()
{
    param(                  
            [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
            [string]$Data
    )

    $parsedData = Import-Csv -Path $Data -Delimiter ";" -Encoding Default

    return $parsedData
}

function Extract-Entry()
{
    param(                  
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        $Entry,
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        [int]$Duration
    )

    #Split date/time and calculate end time
    $split = $Entry.Termin -split '\s+'
    $date = $split[0]
    $starttime = $split[1]
    $splittedTime = $starttime -split ':'
    $startHour = $splittedTime[0]
    $startMinute = $splittedTime[1]
    [string]$endHour = [int]$startHour + $Duration
    $endMinute = $startMinute

    #Property names are defined by google and must not be changed. See: https://support.google.com/calendar/answer/37118?hl=de
    return New-Object -TypeName PSObject -Property @{
            "Subject" = "TT - {0} {1} {2} - {3} {4} {5}" -f $entry.HeimVereinName,$entry.HeimMannschaftAltersklasse,$entry.HeimMannschaftNr,$entry.GastVereinName,$entry.GastMannschaftAltersklasse,$entry.GastMannschaftNr
            "Start Date" = $date
            "Start Time" = $starttime
            "End Date" = $date
            "End Time" = $endHour + ":" + $endMinute
            "Location" = "{0}, {1}, {2} {3}" -f $entry.HalleName,$entry.HalleStrasse,$entry.HallePLZ,$entry.HalleOrt
    }
}

function Write-CsvOutput()
{
    param(                  
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        $Collection,
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        [string]$BaseFileName,
        [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
        [string]$Group
    )

    $Filename = ("{0}_{1}.csv" -f $BaseFileName,$Group)

    $Collection | Export-Csv -Path $Filename -Delimiter ";" -Encoding Default -NoTypeInformation

    Write-Host "Konvertierte Daten ($Group) erfolgreich gespeichert in: $Filename"
}


<#
    Main Execution Workflow
#>

Write-Verbose "Importiere Spielplan"
$parsedData = Parse-Data -Data $Spielplan
Write-Verbose "Konvertiere Daten"
#Initialize output variables
$GroupOutput = @{}
$FullOutput = @()
foreach ($entry in $parsedData)
{
    #Initialize array in hashtable if needed
    if ($GroupOutput[$entry.Staffel] -eq $null)
    {
        $GroupOutput[$entry.Staffel] = @()
    }
    #Add entries to output array / hashtable
    $GroupOutput[$entry.Staffel] += Extract-Entry -Entry $entry -Duration $Spielzeit
    $FullOutput += Extract-Entry -Entry $entry -Duration $Spielzeit
}

Write-Verbose "Exportiere Daten"
Write-CsvOutput -Collection $FullOutput -BaseFileName $Spielplan -Group "Verein"
foreach($group in $GroupOutput.GetEnumerator())
{
    Write-CsvOutput -Collection ($group.value) -BaseFileName $Spielplan -Group $($group.name)
}
